use criterion::{black_box, criterion_group, criterion_main, Criterion};

use microrm::{make_index, Entity, prelude::*};
use rand::prelude::*;
use serde::{Deserialize, Serialize};

#[global_allocator]
static GLOBAL: &stats_alloc::StatsAlloc<std::alloc::System> = &stats_alloc::INSTRUMENTED_SYSTEM;

#[derive(Entity, serde::Serialize, serde::Deserialize)]
pub struct RowIDIndexed {
    val: usize,
}

const MAX_ID: usize = 100;

fn rowid_indexed(c: &mut Criterion) {
    let schema = microrm::Schema::new().entity::<RowIDIndexed>();
    let db = microrm::DB::new_in_memory(schema).unwrap();
    let qi = db.query_interface();

    for i in 0..MAX_ID {
        qi.add(&RowIDIndexed { val: i * i }).unwrap();
    }

    let mut reg = stats_alloc::Region::new(&GLOBAL);

    qi.get().by_id(&RowIDIndexedID(1)).exec().unwrap();

    println!(
        "allocations for single get by_id: {}",
        reg.change_and_reset().allocations
    );

    qi.get().by_id(&RowIDIndexedID(1)).exec().unwrap();

    println!(
        "allocations for second get by_id: {}",
        reg.change_and_reset().allocations
    );

    c.bench_function("select 1", |b| {
        b.iter(|| qi.get().by_id(&RowIDIndexedID(1)).one().unwrap())
    });

    fn random_id() -> RowIDIndexedID {
        let res = RowIDIndexedID((rand::random::<usize>() % MAX_ID) as i64 + 1);
        res
    }

    c.bench_function("select random", |b| {
        b.iter(|| {
            qi.get().by_id(&random_id()).one().unwrap();
        });
    });
}

criterion_group!(benches, rowid_indexed);
criterion_main!(benches);
