#[derive(Debug)]
pub enum Error {
    DBError(sqlite::Error),
    LoadError(String),
    StoreError(String),
    EmptyStoreError,
    CreateError,
    ExecFailure,
}

impl From<sqlite::Error> for Error {
    fn from(e: sqlite::Error) -> Self {
        Self::DBError(e)
    }
}

impl From<Error> for sqlite::Error {
    fn from(e: Error) -> Self {
        match e {
            Error::DBError(e) => e,
            _ => panic!(),
        }
    }
}

impl std::fmt::Display for Error {
    fn fmt(&self, fmt: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
        fmt.write_fmt(format_args!("{:?}", self))
    }
}

impl serde::ser::Error for Error {
    fn custom<T: std::fmt::Display>(msg: T) -> Self {
        Self::StoreError(format!("{}", msg))
    }
}

impl serde::de::Error for Error {
    fn custom<T: std::fmt::Display>(msg: T) -> Self {
        Self::LoadError(format!("{}", msg))
    }
}

impl std::error::Error for Error {}
