use super::build::{DerivedQuery, QueryComponent, QueryPart, StaticVersion};
use super::{Filterable, Resolvable};
use crate::{Entity, Error, QueryInterface};
use std::hash::{Hash, Hasher};
use std::marker::PhantomData;

pub struct Select<'r, 'q, T: Entity> {
    qi: &'r QueryInterface<'q>,
    _ghost: PhantomData<T>,
}

impl<'r, 'q, T: Entity> Select<'r, 'q, T> {
    pub fn new(qi: &'r QueryInterface<'q>) -> Self {
        Self {
            qi,
            _ghost: std::marker::PhantomData,
        }
    }
}

impl<'r, 'q, T: Entity> StaticVersion for Select<'r, 'q, T> {
    type Is = Select<'static, 'static, T>;
}

impl<'r, 'q, T: Entity> QueryComponent for Select<'r, 'q, T> {
    fn derive(&self) -> DerivedQuery {
        DerivedQuery::new().add(
            QueryPart::Root,
            format!("SELECT * FROM {}", T::table_name()),
        )
    }

    fn contribute<H: Hasher>(&self, hasher: &mut H) {
        "select".hash(hasher);
        std::any::TypeId::of::<T>().hash(hasher);
    }

    // next binding point is the first
    fn bind(&self, _stmt: &mut sqlite::Statement<'_>) -> Result<usize, Error> {
        Ok(1)
    }
}

impl<'r, 'q, T: Entity> Filterable<'r, 'q> for Select<'r, 'q, T> {
    type Table = T;
}

impl<'r, 'q, T: Entity> Resolvable<'r, 'q, T> for Select<'r, 'q, T> {
    fn qi(&self) -> &'r QueryInterface<'q> {
        self.qi
    }
}
